
local addonname = "ShotGlass"
local rangePollFrequency = .6
local forceUpdatePollFrequency = 5
local forceUpdateLastUpdate = 0

local UpdateButton, OnShowButton, OnChangeAttribute

local auralist = { 
	-- Druid Auras
	Rejuvenation = {name = "Indicator1", color = {r = .8,g = 0, b = .8, a = 1},},
	Regrowth = {name = "Indicator2", color = {r = .2,g = 1, b = .4, a = 1},},
	
	Lifebloom = {name = "Indicator3", color = {r = .6,g = 1, b = .1, a = 1},},
	["Wild Growth"] = {name = "Indicator3", color = {r = .2,g = 1, b = 1, a = 1},},
	

	
	-- Debuff Types
	Poison = {name = "Indicator5", color = {r = 0,g = .9, b = .1, a = 1},},
	Curse = {name = "Indicator4", color = {r = .9,g = 0, b = .7, a = 1},},
	
	--Disease = {name = "Indicator4", color = {r = .7,g = .7, b = 1, a = 1},},
	--Magic = {name = "Indicator5", color = {r = 0,g = 1, b = .8, a = 1},},
	
	-- Icecrown
	-- Marrowgar
	--"Impaled"
	
	-- Saurfang, "Rune of Blood", "Boiling Blood"
	--"Mark of the Fallen Champion"
	
	-- Festergut
	--"Gas Spore"
	
	-- Rotface
	["Mutated Infection "] = {name = "Indicator4", color = {r = 1,g = 0, b = 0, a = 1},},
	
	-- Putricide
	--"Malleable Goo"
	
	-- Sindragosa 
	--"Unchained Magic"
	
	-- Blood Queen
	["Essence of the Blood Queen"] = {name = "Indicator5", color = {r = 1,g = 0, b = 0, a = 1},},
	["Frenzied Bloodthirst"] = {name = "Indicator5", color = {r = .8,g = .3, b = 0, a = 1},},
	
}

------------------------------------------------------------------------
------------------------------------------------------------------------

local function ResetIndicators(self) 
	local name, indicator
	for name, indicator in pairs(self.Graphics.Indicators) do 
		if type(indicator)=='table' then indicator:Hide() end
	end
end



local function PollOnUpdate(self)
	local _, index, name, count, expiration, mine, auratarget
	local unitid = self.UnitID
	if not unitid then return end

	local now = GetTime()
	if now > self.rangeUpdateTime then self.rangeUpdateTime = now + rangePollFrequency else return end
	if UnitInRange(unitid) then 
		self.Graphics:SetScale(1)
		--self.Graphics:SetAlpha(1)
	else 
		self.Graphics:SetScale(.6)
		--self.Graphics:SetAlpha(.95)
	end
	--[[
	if now > forceUpdateLastUpdate then 
		forceUpdateLastUpdate = forceUpdateLastUpdate + forceUpdatePollFrequency 
		-- Update Everything
	end
	--]]
end

------------------------------------------------------------------------
------------------------------------------------------------------------
local UnitEventHandlers = {}

function UnitEventHandlers.UNIT_DISPLAYPOWER(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then 
		local _, powertype = UnitPowerType(unitid or "")
		if powertype == "MANA" then
			self.Graphics.Power:Show()
			self.Graphics.Power:SetMinMaxValues(0,UnitPowerMax(unitid))
			self.Graphics.Power:SetValue(UnitPower(unitid))
		else
			self.Graphics.Power:Hide()
		end
	end
end

function UnitEventHandlers.UNIT_MANA(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then 
		self.Graphics.Power:SetValue(UnitPower(unitid))
		--print("MANA", unitid, UnitPower(unitid), UnitPowerMax(unitid), self.Graphics.Power:GetValue() )
	end
end
UnitEventHandlers.UNIT_MAXMANA = UnitEventHandlers.UNIT_MANA

function UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unit)
	
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then
		if UnitIsDead(unitid) or UnitIsGhost(unitid) or (not UnitIsConnected(unitid))then 
			self.Graphics.Health:SetAlpha(.05)
			--print("Disabled")
		else 
			self.Graphics.Health:SetAlpha(1)
		end
	end
end
UnitEventHandlers.PARTY_MEMBER_ENABLE = UnitEventHandlers.PARTY_MEMBER_DISABLE
UnitEventHandlers.PLAYER_DEAD = UnitEventHandlers.PARTY_MEMBER_DISABLE



function UnitEventHandlers.UNIT_HEALTH(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then 
		--if UnitHasVehicleUI(unit) then
			--self.Graphics.Health:SetMinMaxValues(0,UnitHealthMax(unitid))
			--self.Graphics.Health:SetValue(UnitHealth(unitid))
		--else
			self.Graphics.Health:SetMinMaxValues(0,UnitHealthMax(unitid))
			self.Graphics.Health:SetValue(UnitHealth(unitid))
		--end
	end
end
UnitEventHandlers.UNIT_MAXHEALTH = UnitEventHandlers.UNIT_HEALTH

function UnitEventHandlers.PLAYER_TARGET_CHANGED(self) 
	local unitid = self.UnitID
	if not unitid then return end
	if UnitIsUnit("target", unitid) then self.Graphics.Border:Show()
	else self.Graphics.Border:Hide() end
end

function UnitEventHandlers.UNIT_THREAT_SITUATION_UPDATE(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	local unitaggro = UnitThreatSituation(unitid)
	if unitaggro and unitaggro > 1 then 
		--self.Graphics:SetScale(1.3)
		self.Graphics.Aggro:Show() 
	else 
		--self.Graphics:SetScale(1.0)
		self.Graphics.Aggro:Hide() 
	end
end



function UnitEventHandlers.UNIT_AURA(self, unit)
	local index, name, debuffType, caster , stacks, _
	local unitid = self.UnitID
	if not unitid then return end
	
	if unit == unitid then 
		ResetIndicators(self)
		-- Check Buffs
		for index = 1, 40 do
			name, _, _, stacks, _, _, _, caster = UnitAura(unitid,index,"PLAYER")
			if not name then break end
			auratarget = auralist[name] 
			if auratarget then 
				
				local color = auratarget.color
				self.Graphics.Indicators[auratarget.name].Image:SetVertexColor(color.r,color.g,color.b,color.a)
				if (stacks > 0) and (caster == "player") then self.Graphics.Indicators[auratarget.name].Text:SetText(stacks) 
				else self.Graphics.Indicators[auratarget.name].Text:SetText("") end
				self.Graphics.Indicators[auratarget.name]:Show()
				
			end
		end
		-- Check Debuffs
		for index = 1, 40 do
			name, _, _, _, debuffType = UnitDebuff(unitid,index)
			if not name then break end
			auratarget = auralist[name] or auralist[debuffType]
			if auratarget then 
				local color = auratarget.color
				self.Graphics.Indicators[auratarget.name].Image:SetVertexColor(color.r,color.g,color.b,color.a)
				self.Graphics.Indicators[auratarget.name]:Show()
			end
		end
		
	end
end

-- [[
--RESURRECT_REQUEST

function UnitEventHandlers.UNIT_PORTRAIT_UPDATE(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	UnitEventHandlers.UNIT_DISPLAYPOWER(self, unitid)
	UnitEventHandlers.UNIT_AURA(self, unitid)
	UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)	
	UnitEventHandlers.UNIT_HEALTH(self, unitid)
end
UnitEventHandlers.PLAYER_UNGHOST = UnitEventHandlers.UNIT_PORTRAIT_UPDATE
UnitEventHandlers.PLAYER_ALIVE = UnitEventHandlers.UNIT_PORTRAIT_UPDATE

function UnitEventHandlers.UNIT_FLAGS(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	UnitEventHandlers.UNIT_AURA(self, unitid)
	UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)	
end

-- UNIT_FLAGS - Stunned, Feared, etc.  -- disabled, health, aura

--]]

--[[


PARTY_MEMBERS_CHANGED
RAID_ROSTER_UPDATE
READY_CHECK
READY_CHECK_CONFIRM
UNIT_CLASSIFICATION_CHANGED		-- Elite to non-elite
UNIT_DYNAMIC_FLAGS
UNIT_ENTERED_VEHICLE
UNIT_EXITED_VEHICLE
UNIT_ENTERING_VEHICLE
UNIT_EXITING_VEHICLE
UNIT_FLAGS

PLAYER_REGEN_DISABLED
PLAYER_REGEN_ENABLED

PLAYER_UNGHOST


--]]
------------------------------------------------------------------------
------------------------------------------------------------------------
local function ApplyButtonStyle(self) end
local function AbbrevName(text) return text:gsub("(%u%l%l%l%l)(.*)", "%1") end

local GlobalToName = {}

function UpdateButton(self, unitid)

		local class, color, Graphics
		GlobalToName[UnitGUID(unitid)] = UnitName(unitid)
		
		self.Nextupdate = 0
		Graphics = self.Graphics
		-- Colors
		class = select(2, UnitClass(unitid)) or "WARRIOR"
		color = RAID_CLASS_COLORS[class]
		-- Name
		Graphics.Text:SetText(AbbrevName(UnitName(unitid)))
		--Graphics.Text:SetTextColor(color.r, color.g, color.b, 1)
		Graphics.Text:SetTextColor(color.r*.5, color.g*.5, color.b*.5, 1)
		-- Health
		--Graphics.Health:SetStatusBarColor(0.05, 0.05, 0.05, .9)
		Graphics.Health:SetStatusBarColor(0.05, 0.05, 0.05, 1)
		Graphics.Health:SetBackdropColor(color.r, color.g, color.b, 1)
		Graphics.Health:SetMinMaxValues(0,UnitHealthMax(unitid))
		-- Power		
		UnitEventHandlers.UNIT_DISPLAYPOWER(self, unitid)
		UnitEventHandlers.UNIT_AURA(self, unitid)
		UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)	
		UnitEventHandlers.UNIT_HEALTH(self, unitid)
		
end

function OnShowButton(self)
	local unitid = self:GetAttribute("unit") 
	self.UnitID = unitid
	if unitid then 
		if not InCombatLockdown() then
			self:SetAttribute('*type1','target') -- Set button to Click target behavior
		end
		UpdateButton(self, unitid)
	end
end

function OnChangeAttribute(self, attribute, value)
	local unitid = self:GetAttribute("unit")
	if attribute == "unit" and unitid then UpdateButton(self, unitid) end
end

------------------------------------------------------------------------
local function OnCreateButton(self, other)
	--print("Button", self, "Parent", self:GetParent())
	local Graphics
	self:SetAttribute('initial-width',47)
	self:SetAttribute('initial-height',47)
	self:EnableMouse(true)
	
	
	self.info = {}
	
	-- Create Base Frames
	self.AuraWatcher = CreateFrame("Frame", nil, self)
	self.Graphics = CreateFrame("Frame", nil, self)
	self.auraUpdateTime = 0
	self.rangeUpdateTime = 0
	
	Graphics = self.Graphics
	Graphics:SetPoint("CENTER", self)
	Graphics:SetWidth(64)
	Graphics:SetHeight(64)
		
	-- Health Bar 
	Graphics.Health = CreateFrame("StatusBar", nil, Graphics)
	--Graphics.Health = CreateShotGlassStatusbar(Graphics)
	Graphics.Health:SetStatusBarTexture("Interface\\Addons\\ShotGlass\\Art\\HealthStatusbar")
	Graphics.Health:SetBackdrop({bgFile = "Interface\\Addons\\ShotGlass\\Art\\HealthStatusbar"});
	Graphics.Health:SetOrientation("VERTICAL")
	Graphics.Health:SetPoint("CENTER", 0, -1)
	Graphics.Health:SetFrameLevel(Graphics:GetFrameLevel()-1)
	Graphics.Health:SetWidth(36)
	Graphics.Health:SetHeight(34)
	
	-- Power (Mana) Bar
	Graphics.Power = CreateFrame("StatusBar", nil, Graphics)
	--Graphics.Power = CreateShotGlassStatusbar(Graphics)
	Graphics.Power:SetStatusBarTexture("Interface\\Addons\\ShotGlass\\Art\\ManaStatusbar")
	Graphics.Power:SetBackdrop({bgFile = "Interface\\Addons\\ShotGlass\\Art\\ManaStatusbar"});
	Graphics.Power:SetOrientation("VERTICAL")
	Graphics.Power:SetPoint("CENTER", 16, 7)
	Graphics.Power:SetHeight(28)	
	Graphics.Power:SetWidth(7)	
	Graphics.Power:SetBackdropColor(0, 0, 0, 1)
	Graphics.Power:SetStatusBarColor(4/255, 78/255, 141/255, 1)

	Graphics.Power.Border = Graphics.Power:CreateTexture(nil, "OVERLAY")
	Graphics.Power.Border:SetTexture("Interface\\Addons\\ShotGlass\\Art\\ManaOverlay")
	Graphics.Power.Border:SetPoint("TOPLEFT", -1,1)	
	Graphics.Power.Border:SetHeight(32)	
	Graphics.Power.Border:SetWidth(16)	

	-- Create Name Text Field
	Graphics.Text = Graphics:CreateFontString(nil, "ARTWORK")
	Graphics.Text:SetFont("Interface\\Addons\\ShotGlass\\Art\\LiberationSans-Regular.ttf", 10)
	Graphics.Text:SetPoint("BOTTOM", Graphics.Health, "BOTTOM", 0, 1)
	Graphics.Text:SetWidth(50)
	Graphics.Text:SetHeight(13)
	Graphics.Text:SetShadowColor(0,0,0,1) 
	Graphics.Text:SetShadowOffset(1,-1) 
	
	--[[
	-- Create Mouseover Highlight
	Graphics.Glow = Graphics:CreateTexture(nil, "HIGHLIGHT")
	Graphics.Glow:SetTexture("Interface/TARGETINGFRAME/UI-StatusBar")
	Graphics.Glow:SetVertexColor(.5,.5,.5,.5)
	Graphics.Glow:SetBlendMode("ADD")
	Graphics.Glow:SetAllPoints(Graphics.Health)
	--]]
	
	-- Overlay Artwork
	Graphics.Overlay = Graphics:CreateTexture(nil, "ARTWORK")
	Graphics.Overlay:SetTexture("Interface\\Addons\\ShotGlass\\Art\\HealthOverlay")
	Graphics.Overlay:SetAllPoints(Graphics)
	
	-- Aggro/Outline Artwork
	Graphics.Aggro = Graphics:CreateTexture(nil, "OVERLAY")
	Graphics.Aggro:SetTexture("Interface\\Addons\\ShotGlass\\Art\\Aggro")
	Graphics.Aggro:SetVertexColor(1,0,0,1)
	Graphics.Aggro:SetAllPoints()
	Graphics.Aggro:Hide()

	-- Indicators Table
	Graphics.IndicatorLayer = CreateFrame("Frame", nil, Graphics)
	Graphics.IndicatorLayer:SetAllPoints(Graphics)
	Graphics.IndicatorLayer:Show()
	
	Graphics.Indicators = {}
	local Indicators = Graphics.Indicators
	local Indicator, IndicatorIndex
	
	for index = 1, 6 do
		IndicatorIndex = "Indicator"..index
		Indicators[IndicatorIndex] = CreateFrame("Frame", nil, Graphics.IndicatorLayer)
		Indicator = Indicators[IndicatorIndex]
		Indicator:SetWidth(16)
		Indicator:SetHeight(16)
		--
		Indicator.Image = Indicator:CreateTexture(nil, "ARTWORK")
		--Indicator.Image:SetTexture("Interface\\Addons\\ShotGlass\\Art\\DebuffDot")
		Indicator.Image:SetAllPoints()
		--
		Indicator.Text = Indicator:CreateFontString(nil, "OVERLAY")
		Indicator.Text:SetFont("Interface\\Addons\\ShotGlass\\Art\\LiberationSans-Regular.ttf", 7, "OUTLINE")
		Indicator.Text:SetPoint("TOP", Indicator, "BOTTOM", 0, 7)
		Indicator.Text:SetWidth(12)
		Indicator.Text:SetHeight(12)
		--Indicator.Text:SetText("2")
		--
		Indicator:Hide()	
	end

	Indicators.Indicator1:SetPoint("TOPRIGHT", -10-8, -7)
	Indicators.Indicator2:SetPoint("TOPRIGHT", -19-8, -7)
	Indicators.Indicator3:SetPoint("TOPRIGHT", -28-8, -7)
	
	Indicators.Indicator4:SetPoint("BOTTOM", -9, 4)
	Indicators.Indicator5:SetPoint("BOTTOM", 0, 4)
	Indicators.Indicator6:SetPoint("BOTTOM", 9, 4)
	
	--Indicators.Indicator4.Image:SetTexture("Interface\\Addons\\ShotGlass\\Art\\DebuffDot")
	
	Indicators.Indicator1.Image:SetTexture("Interface\\Addons\\ShotGlass\\Art\\DebuffDot")
	Indicators.Indicator2.Image:SetTexture("Interface\\Addons\\ShotGlass\\Art\\DebuffDot")
	Indicators.Indicator3.Image:SetTexture("Interface\\Addons\\ShotGlass\\Art\\DebuffDot")
	
	Indicators.Indicator4.Image:SetTexture("Interface\\Addons\\ShotGlass\\Art\\DebuffArrow")
	Indicators.Indicator5.Image:SetTexture("Interface\\Addons\\ShotGlass\\Art\\DebuffArrow")
	Indicators.Indicator6.Image:SetTexture("Interface\\Addons\\ShotGlass\\Art\\DebuffArrow")
	
	-- Border/Selection Box Artwork
	Graphics.Border = Graphics:CreateTexture(nil, "ARTWORK")
	Graphics.Border:SetTexture("Interface\\Addons\\ShotGlass\\Art\\Select")
	Graphics.Border:SetAllPoints()
	Graphics.Border:Hide()
	
	-- Scripts
	self:SetScript("OnShow", OnShowButton)
	self:SetScript("OnAttributeChanged", OnChangeAttribute)
	self:SetScript("OnEvent", function(self, event, ...) UnitEventHandlers[event](self, ...) end)
	self:SetScript("OnUpdate", PollOnUpdate)
	-- Register Events
	for eventname, _ in pairs(UnitEventHandlers) do self:RegisterEvent(eventname) end
end

--[[

local function CallEventHandlers()
	for 
	-- UnitEventHandlers[event](self, ...)
end
--]]

local function CreateSecureGroupHeader()
	local parent = CreateFrame("Frame", addonname.."_Main", UIParent)
	
	--[[
	parent.Backdrop = parent:CreateTexture(nil, "BACKGROUND")
	parent.Backdrop:SetTexture("Interface\\Addons\\ShotGlass\\Art\\Corner")
	parent.Backdrop:SetHeight(32)
	parent.Backdrop:SetWidth(32)
	parent.Backdrop:SetPoint("BOTTOMLEFT", -3, -3)
	--]]
	
	local raid = CreateFrame("Frame", addonname.."_RaidHeader", parent, "SecureGroupHeaderTemplate")
	--print("Raid Frame", raid)
	
	-- RAIDFRAME TEMPLATE
	raid:SetAttribute("template", "SecureUnitButtonTemplate")
	raid:SetAttribute("templateType", "Button")
	raid:SetAttribute("groupFilter", "1,2,3,4,5,6,7,8")
	raid:SetAttribute("showRaid", true)
	raid:SetAttribute("showPlayer", true)
	raid:SetAttribute("showSolo", true)
	raid.initialConfigFunction = OnCreateButton
	raid:SetAttribute('point','LEFT')
	raid:SetAttribute('columnAnchorPoint','TOP')
	raid:SetAttribute('yOffset', 0)
	--raid:SetAttribute('xOffset', 6)
	--raid:SetAttribute('columnSpacing',5)
	
	raid:SetAttribute('xOffset', 2)
	raid:SetAttribute('columnSpacing',2)
	
	raid:SetAttribute('maxColumns',8)
	--raid:SetAttribute('maxColumns',2)
	raid:SetAttribute('unitsPerColumn',5)
	raid:SetMovable(true)
	raid:SetPoint("LEFT", UIParent, "LEFT", 100, 0)
	
	--raid:SetScale(3)
	raid:SetScale(1.2)
	
	local padding = 8
	parent:SetPoint("TOP", raid, "TOP", 0, padding)
	parent:SetPoint("BOTTOM", raid, "BOTTOM", 0, -padding)
	parent:SetPoint("LEFT", raid, "LEFT", -padding, 0)
	parent:SetPoint("RIGHT", raid, "RIGHT", padding, 0)
		
	
	parent:EnableMouse(true)
	parent:SetScript("OnMouseDown",function() raid:StartMoving() end)
	parent:SetScript("OnMouseUp",function() raid:StopMovingOrSizing() end)
	
	parent.raid = raid
	return parent
end

------------------------------------------------------------------------ Init
local RaidFrame = CreateSecureGroupHeader()
RaidFrame.raid:SetClampedToScreen(true)
RaidFrame.raid:Show()

ShotGlassRaidFrame = RaidFrame
--parent:EnableMouse(true)
--/run ShotGlassRaidFrame:EnableMouse(false)
--/run ShotGlassRaidFrame.raid:ClearAllPoints(); ShotGlassRaidFrame.raid:SetPoint("CENTER", UIParent)
--/run ShotGlassRaidFrame.raid:SetPoint("CENTER", UIParent)
